@@ -1,9 +1,12 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 import datetime
+import decimal
 import enum
 import functools
 import inspect
+import ipaddress
+import pathlib
 from operator import attrgetter
 from typing import (
     Any,
@@ -43,6 +46,7 @@ str
 __all__ = (
     "BUILTIN_TYPES",
     "ObjectT",
+    "isbuiltininstance",
     "isbuiltintype",
     "isbuiltinsubtype",
     "isclassvartype",
@@ -59,7 +63,11 @@ __all__ = (
     "isnamedtuple",
     "isoptionaltype",
     "isreadonly",
+    "issimpleattribute",
     "isstrict",
+    "isstdlibinstance",
+    "isstdlibtype",
+    "isstdlibsubtype",
     "issubclass",
     "istypeddict",
     "istypedtuple",
@@ -86,6 +94,19 @@ BUILTIN_TYPES = frozenset(
         type(None),
     )
 )
+BUILTIN_TYPES_TUPLE = tuple(BUILTIN_TYPES)
+STDLIB_TYPES = frozenset(
+    (
+        *BUILTIN_TYPES,
+        datetime.datetime,
+        datetime.date,
+        decimal.Decimal,
+        pathlib.Path,
+        ipaddress.IPv4Address,
+        ipaddress.IPv6Address,
+    )
+)
+STDLIB_TYPES_TUPLE = tuple(STDLIB_TYPES)
 
 
 @functools.lru_cache(maxsize=None)
@@ -110,7 +131,7 @@ def isbuiltintype(obj: Type[ObjectT]) -> bool:
     Examples
     --------
 
-import typic
+    >>> import typic
     >>> from typing import NewType, Mapping
     >>> typic.isbuiltintype(str)
     True
@@ -129,6 +150,14 @@ import typic
     )
 
 
+@functools.lru_cache(maxsize=None)
+def isstdlibtype(obj: Type[ObjectT]) -> bool:
+    return (
+        util.resolve_supertype(obj) in STDLIB_TYPES
+        or util.resolve_supertype(type(obj)) in STDLIB_TYPES
+    )
+
+
 @functools.lru_cache(maxsize=None)
 def isbuiltinsubtype(t: Type[ObjectT]) -> bool:
     """Check whether the provided type is a subclass of a builtin-type.
@@ -154,7 +183,20 @@ def isbuiltinsubtype(t: Type[ObjectT]) -> bool:
     >>> typic.isbuiltintype(Mapping)
     False
     """
-    return issubclass(util.resolve_supertype(t), (*BUILTIN_TYPES,))
+    return issubclass(util.resolve_supertype(t), BUILTIN_TYPES_TUPLE)
+
+
+@functools.lru_cache(maxsize=None)
+def isstdlibsubtype(t: Type[ObjectT]) -> bool:
+    return issubclass(util.resolve_supertype(t), STDLIB_TYPES_TUPLE)
+
+
+def isbuiltininstance(o: ObjectT) -> bool:
+    return _isinstance(o, BUILTIN_TYPES_TUPLE)
+
+
+def isstdlibinstance(o: ObjectT) -> bool:
+    return _isinstance(o, STDLIB_TYPES_TUPLE)
 
 
 @functools.lru_cache(maxsize=None)
@@ -171,7 +213,7 @@ def isoptionaltype(obj: Type[ObjectT]) -> bool:
     Examples
     --------
 
-import typic
+    >>> import typic
     >>> from typing import Optional, Union, Dict
     >>> typic.isoptionaltype(Optional[str])
     True
@@ -200,7 +242,7 @@ def isreadonly(obj: Type[ObjectT]) -> bool:
     Examples
     --------
 
-import typic
+    >>> import typic
     >>> from typing import NewType
     >>> typic.isreadonly(typic.ReadOnly[str])
     True
@@ -217,7 +259,7 @@ def isfinal(obj: Type[ObjectT]) -> bool:
     Examples
     --------
 
-import typic
+    >>> import typic
     >>> from typing import NewType
     >>> from typic.compat import Final
     >>> typic.isfinal(Final[str])
@@ -239,7 +281,7 @@ def iswriteonly(obj: Type[ObjectT]) -> bool:
     Examples
     --------
 
-import typic
+    >>> import typic
     >>> from typing import NewType
     >>> typic.iswriteonly(typic.WriteOnly[str])
     True
@@ -260,7 +302,7 @@ def isstrict(obj: Type[ObjectT]) -> bool:
     Examples
     --------
 
-import typic
+    >>> import typic
     >>> from typing import NewType
     >>> typic.iswriteonly(typic.WriteOnly[str])
     True
@@ -281,7 +323,7 @@ def isdatetype(obj: Type[ObjectT]) -> bool:
     Examples
     --------
 
-import typic
+    >>> import typic
     >>> import datetime
     >>> from typing import NewType
     >>> typic.isdatetype(datetime.datetime)
@@ -310,7 +352,7 @@ def iscollectiontype(obj: Type[ObjectT]):
     Examples
     --------
 
-import typic
+    >>> import typic
     >>> from typing import Collection, Mapping, NewType
     >>> typic.iscollectiontype(Collection)
     True
@@ -340,7 +382,7 @@ def ismappingtype(obj: Type[ObjectT]):
     Examples
     --------
 
-import typic
+    >>> import typic
     >>> from typing import Mapping, Dict, DefaultDict, NewType
     >>> typic.ismappingtype(Mapping)
     True
@@ -376,7 +418,8 @@ def isenumtype(obj: Type[ObjectT]) -> bool:
     Examples
     --------
 
-import typic    >>> import enum
+    >>> import typic
+    >>> import enum
     >>>
     >>> class FooNum(enum.Enum): ...
     ...
@@ -628,7 +671,7 @@ def isnamedtuple(obj: Type[ObjectT]) -> bool:
 
 
 def isproperty(obj) -> bool:
-    return type(obj).__name__ in {"property", "cached_property"}
+    return obj.__class__.__name__ in {"property", "cached_property"}
 
 
 _ATTR_CHECKS = (inspect.isclass, inspect.isroutine, isproperty)
