@@ -89,6 +89,7 @@ def unique(seq: Sequence, *, ret_type: Type[Union[list, tuple]] = list) -> Seque
         return unique_slow(seq, ret_type=ret_type)
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class ArrayConstraints(BaseConstraints):
     """Specific constraints pertaining to a sized, array-like type.
@@ -124,10 +125,10 @@ class ArrayConstraints(BaseConstraints):
         # Build the code.
         # Only make it unique if we have to. This preserves order as well.
         if self.unique is True and util.origin(self.type) not in {set, frozenset}:
-            func.l(f"{self.VAL} = __unique({self.VAL})", __unique=unique)
+            func.l(f"{self.VALUE} = __unique({self.VALUE})", __unique=unique)
         # Only get the size if we have to.
         if {self.max_items, self.min_items} != {None, None}:
-            func.l(f"size = len({self.VAL})")
+            func.l(f"size = len({self.VALUE})")
         # Get the validation checks and context
         asserts: List[str] = []
         context: Dict[str, Any] = {}
@@ -139,12 +140,17 @@ class ArrayConstraints(BaseConstraints):
         if self.values:
             o = util.origin(self.type)
             itval = "__item_validator"
-            ctx = {itval: self.values.validate, o.__name__: o}
-            field = f"'.'.join(({self.VALTNAME}, str(i)))"
+            ctx = {
+                itval: self.values.validate,
+                o.__name__: o,
+                "_lazy_repr": util.collectionrepr,
+            }
+            r = "i" if issubclass(self.type, Sequence) else "x"
+            field = f"_lazy_repr({self.FNAME}, {r})"
             func.l(
-                f"{self.VAL} = "
+                f"{self.VALUE} = "
                 f"{o.__name__}("
-                f"({itval}(x, field={field}) for i, x in enumerate({self.VAL}))"
+                f"({itval}(x, field={field}) for i, x in enumerate({self.VALUE}))"
                 f")",
                 **ctx,
             )
@@ -163,6 +169,7 @@ class ArrayConstraints(BaseConstraints):
         return {x: y for x, y in schema.items() if y is not None}
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class ListContraints(ArrayConstraints):
     """Specific constraints pertaining to a :py:class:`list`."""
@@ -170,6 +177,7 @@ class ListContraints(ArrayConstraints):
     type: ClassVar[Type[list]] = list
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class TupleContraints(ArrayConstraints):
     """Specific constraints pertaining to a :py:class:`tuple`."""
@@ -177,6 +185,7 @@ class TupleContraints(ArrayConstraints):
     type: ClassVar[Type[tuple]] = tuple
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class SetContraints(ArrayConstraints):
     """Specific constraints pertaining to a :py:class:`set`."""
@@ -185,6 +194,7 @@ class SetContraints(ArrayConstraints):
     unique: bool = True
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class FrozenSetConstraints(ArrayConstraints):
     """Specific constraints pertaining to a :py:class:`frozenset`."""
