@@ -22,10 +22,8 @@ from typing import (
 )
 
 from typic import gen, util
-from typic.strict import STRICT_MODE
 from .error import ConstraintValueError
 
-
 if TYPE_CHECKING:  # pragma: nocover
     from typic.constraints.factory import ConstraintsT  # noqa: F401
 
@@ -39,7 +37,7 @@ __all__ = (
 
 VT = TypeVar("VT")
 """A generic type-var for values passed to a validator."""
-ValidatorT = Callable[[VT], Tuple[bool, VT]]
+ValidatorT = Callable[[VT, Optional[str]], Tuple[bool, VT]]
 """The expected signature of a value validator."""
 ChecksT = List[str]
 ContextT = Dict[str, Any]
@@ -48,33 +46,56 @@ _T = TypeVar("_T")
 
 class __AbstractConstraints(abc.ABC):
     type: ClassVar[Type[Any]]
+    VALUE = "value"
+    VALTNAME = "valtname"
+    FIELD = "field"
+    FNAME = "fieldname"
+
+    __slots__ = ("__dict__",)
 
     def __post_init__(self):
         self.validator
 
-    __repr = util.cached_property(util.filtered_repr)
-
-    def __repr__(self) -> str:
-        return self.__repr
-
     @util.cached_property
     def __str(self) -> str:
-        fields = [f"type={self.type_name}"]
+        fields = [f"type={self.type_qualname}"]
         for f in dataclasses.fields(self):
+            if f.name == "type":
+                continue
+
             val = getattr(self, f.name)
             if (val or val in {False, 0}) and f.repr:
-                fields.append(f"{f.name}={val}")
+                fields.append(f"{f.name}={val!r}")
         return f"({', '.join(fields)})"
 
     def __str__(self) -> str:
         return self.__str
 
+    def __repr__(self):
+        return self.__str
+
+    def define(self, block: gen.Block, name: str) -> gen.Block:
+        return block.f(
+            name,
+            block.param(self.VALUE, annotation="VT"),
+            block.param(
+                "field",
+                annotation=str,
+                kind=gen.ParameterKind.KEYWORD_ONLY,  # type: ignore
+                default=None,
+            ),
+        )
+
     @util.cached_property
     def type_name(self) -> str:
         return util.get_name(self.type)
 
+    @util.cached_property
+    def type_qualname(self) -> str:
+        return util.get_qualname(self.type)
+
     def _get_validator_name(self) -> str:
-        return f"validator_{util.hexhash(self)}"
+        return util.get_defname("validator", self)
 
     @util.cached_property
     @abc.abstractmethod
@@ -98,7 +119,11 @@ class __AbstractConstraints(abc.ABC):
             An error inheriting from :py:class:`SyntaxError` indicating the constraint
             configuration is invalid.
         """
-        valid, value = self.validator(value)
+        try:
+            valid, value = self.validator(value, field=field)
+        except AttributeError:
+            valid, value = False, value
+
         if not valid:
             field = f"{field}:" if field else "Given"
             raise ConstraintValueError(
@@ -133,6 +158,7 @@ class InstanceCheck(enum.IntEnum):
     """
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)  # type: ignore
 class BaseConstraints(__AbstractConstraints):
     """A base constraints object. Shouldn't be used directly.
@@ -162,9 +188,6 @@ class BaseConstraints(__AbstractConstraints):
     """
     name: Optional[str] = None
 
-    VAL = "val"
-    VALTNAME = "valtname"
-
     def _build_validator(
         self, func: gen.Block
     ) -> Tuple[ChecksT, ContextT]:  # pragma: nocover
@@ -174,38 +197,39 @@ class BaseConstraints(__AbstractConstraints):
         if checks:
             check = " and ".join(checks)
             func.l(f"valid = {check}", **context)
-            func.l(f"return valid, {self.VAL}")
+            func.l(f"return valid, {self.VALUE}")
         else:
-            func.l(f"return True, {self.VAL}", **context)
+            func.l(f"return True, {self.VALUE}", **context)
 
     def _compile_validator(self) -> ValidatorT:
         func_name = self._get_validator_name()
         origin = util.origin(self.type)
         type_name = self.type_name
         with gen.Block() as main:
-            with main.f(func_name, main.param(self.VAL)) as f:
+            with self.define(main, func_name) as f:
                 # This is a signal that -*-anything can happen...-*-
                 if origin in {Any, Signature.empty}:
-                    f.l(f"return True, {self.VAL}")
+                    f.l(f"return True, {self.VALUE}")
                     return main.compile(name=func_name)
                 f.l(f"{self.VALTNAME} = {type_name!r}")
+                f.l(f"{self.FNAME} = {self.VALTNAME} if field is None else field")
                 # Short-circuit validation if the value isn't the correct type.
                 if self.instancecheck == InstanceCheck.IS:
-                    line = f"if isinstance({self.VAL}, {type_name}):"
+                    line = f"if isinstance({self.VALUE}, {type_name}):"
                     if self.nullable:
                         line = (
-                            f"if {self.VAL} is None "
-                            f"or isinstance({self.VAL}, {type_name}):"
+                            f"if {self.VALUE} is None "
+                            f"or isinstance({self.VALUE}, {type_name}):"
                         )
                     with f.b(line, **{type_name: self.type}) as b:  # type: ignore
-                        b.l(f"return True, {self.VAL}")
+                        b.l(f"return True, {self.VALUE}")
                 else:
                     if self.nullable:
-                        with f.b(f"if {self.VAL} is None:") as b:
-                            b.l(f"return True, {self.VAL}")
-                    line = f"if not isinstance(val, {type_name}):"
+                        with f.b(f"if {self.VALUE} is None:") as b:
+                            b.l(f"return True, {self.VALUE}")
+                    line = f"if not isinstance({self.VALUE}, {type_name}):"
                     with f.b(line, **{type_name: self.type}) as b:  # type: ignore
-                        b.l(f"return False, {self.VAL}")
+                        b.l(f"return False, {self.VALUE}")
                 checks, context = self._build_validator(f)
                 # Write the line.
                 self._set_return(func=f, checks=checks, context=context)
@@ -225,6 +249,7 @@ class BaseConstraints(__AbstractConstraints):
         return validator
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class MultiConstraints(__AbstractConstraints):
     """A container for multiple constraints for a single field."""
@@ -259,6 +284,14 @@ class MultiConstraints(__AbstractConstraints):
     def type(self) -> Tuple[Type, ...]:  # type: ignore
         return (*self.__type(),)
 
+    @util.cached_property
+    def type_name(self) -> str:
+        return f"({', '.join(util.get_name(t) for t in self.type)})"
+
+    @util.cached_property
+    def type_qualname(self) -> str:
+        return f"({', '.join(util.get_qualname(t) for t in self.type)})"
+
     @util.cached_property
     def validator(self) -> ValidatorT:
         """Accessor for the generated multi-validator.
@@ -268,30 +301,30 @@ class MultiConstraints(__AbstractConstraints):
         If a value does not match any origin-type, as reported by :py:func:`typic.origin`,
         then we will report the value as invalid.
         """
-        vmap: Dict[Type, ValidatorT] = {
-            util.origin(c.type): c.validator for c in self.constraints
-        }
+        vmap = util.TypeMap(
+            {util.origin(c.type): c.validator for c in self.constraints}
+        )
         if vmap:
             if self.nullable:
 
-                def multi_validator(val: VT) -> Tuple[bool, VT]:
-                    if val is None:
-                        return True, val
-                    t = type(val)
-                    return vmap[t](val) if t in vmap else (False, val)
+                def multi_validator(value: VT, *, field: str = None) -> Tuple[bool, VT]:
+                    if value is None:
+                        return True, value
+                    v: Optional[ValidatorT] = vmap.get_by_parent(type(value), None)
+                    return v(value, field=field) if v else (False, value)  # type: ignore
 
             else:
 
-                def multi_validator(val: VT) -> Tuple[bool, VT]:
-                    t = type(val)
-                    return vmap[t](val) if t in vmap else (False, val)
+                def multi_validator(value: VT, *, field: str = None) -> Tuple[bool, VT]:
+                    v: Optional[ValidatorT] = vmap.get_by_parent(type(value), None)
+                    return v(value, field=field) if v else (False, value)  # type: ignore
 
         else:
 
-            def multi_validator(val: VT) -> Tuple[bool, VT]:
-                return True, val
+            def multi_validator(value: VT, *, field: str = None) -> Tuple[bool, VT]:
+                return True, value
 
-        return multi_validator
+        return multi_validator  # type: ignore
 
     def for_schema(self, *, with_type: bool = False) -> dict:
         scheme: dict = {
@@ -302,6 +335,7 @@ class MultiConstraints(__AbstractConstraints):
         return scheme
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class TypeConstraints(__AbstractConstraints):
     """A container for simple types. Validation is limited to instance checks.
@@ -311,7 +345,9 @@ class TypeConstraints(__AbstractConstraints):
     >>> import typic
     >>> tc = typic.get_constraints(typic.AbsoluteURL)
     >>> tc.validate("http://foo.bar")
-    'http://foo.bar'
+    Traceback (most recent call last):
+        ...
+    typic.constraints.error.ConstraintValueError: Given value <'http://foo.bar'> fails constraints: (type=AbsoluteURL, nullable=False)
     >>> tc = typic.get_constraints(typic.AbsoluteURL, nullable=True)
     >>> tc.validate(None)
 
@@ -324,36 +360,20 @@ class TypeConstraints(__AbstractConstraints):
     """Whether this constraint can allow null values."""
     name: Optional[str] = None
 
-    __str = util.cached_property(util.filtered_str)
-
-    def __str__(self) -> str:
-        return self.__str
-
-    @property
-    def coerce(self) -> bool:
-        """Whether coercion can be used in lieu of validation.
-
-        There are a large number of types provided by the standard lib which are much
-        more strict than primitives. These can be relied upon to error out if given
-        invalid inputs, so we can signal upstream to use this method.
-        """
-        return not STRICT_MODE
-
     @util.cached_property
     def validator(self) -> ValidatorT:
         ns = dict(__t=self.type, VT=VT)
         func_name = self._get_validator_name()
         with gen.Block(ns) as main:
-            with main.f(func_name, main.param("value", annotation="VT")) as f:
+            with self.define(main, func_name) as f:
                 # Standard instancecheck is default.
                 check = "isinstance(value, __t)"
                 retval = "value"
-                # This is just a pass-through, the coercer does the real work.
-                if self.coerce:
-                    check = "True"
                 # Have to allow nulls if its nullable.
-                elif self.nullable:
+                if self.nullable:
                     check = "(value is None or isinstance(value, __t))"
+                elif self.type is Any:
+                    check = "True"
                 f.l(f"{gen.Keyword.RET} {check}, {retval}")
 
         validator: ValidatorT = main.compile(name=func_name, ns=ns)
@@ -365,6 +385,7 @@ class TypeConstraints(__AbstractConstraints):
         return {}
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class EnumConstraints(__AbstractConstraints):
     type: Type[enum.Enum]  # type: ignore
@@ -391,9 +412,9 @@ class EnumConstraints(__AbstractConstraints):
         ns = dict(__t=self.type, VT=VT, __values=(*self.type,))
         func_name = self._get_validator_name()
         with gen.Block(ns) as main:
-            with main.f(func_name, main.param("value", annotation="VT")) as f:
+            with self.define(main, func_name) as f:
                 if self.nullable:
-                    with f.b(f"if value is None:") as b:
+                    with f.b("if value is None:") as b:
                         b.l(f"{gen.Keyword.RET} value")
                 # This is O(N), but so is casting to the enum
                 # And handling a ValueError is an order of magnitude heavier
