@@ -1,5 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
+import dataclasses
 import enum
 import functools
 import inspect
@@ -29,6 +30,7 @@ from typic.checks import (
     istypeddict,
     isbuiltinsubtype,
     isnamedtuple,
+    should_unwrap,
 )
 from typic.types import dsn, email, frozendict, path, secret, url
 from typic.util import (
@@ -167,7 +169,9 @@ _SIMPLE_CONSTRAINTS = TypeMap(
 def _from_simple_type(
     t: Type[SimpleT], *, nullable: bool = False, name: str = None
 ) -> SimpleConstraintsT:
-    constr_class = cast(Type[SimpleConstraintsT], _SIMPLE_CONSTRAINTS.get_by_parent(t))
+    constr_class = cast(
+        Type[SimpleConstraintsT], _SIMPLE_CONSTRAINTS.get_by_parent(origin(t))
+    )
     return constr_class(nullable=nullable, name=name)
 
 
@@ -237,20 +241,30 @@ def _from_class(
     items: Optional[
         frozendict.FrozenDict[Hashable, ConstraintsT]
     ] = frozendict.FrozenDict(_resolve_params(**params)) or None
-    required = frozenset((x for x, y in params.items() if y.default is y.empty))
-    total = getattr(t, "__total__", None) or not required
-    kwargs = dict(
-        type=t,
-        nullable=nullable,
-        name=name,
-        required_keys=required,
-        items=items,
-        total=total,
+    required = frozenset(
+        (
+            pname
+            for pname, p in params.items()
+            if (
+                p.kind not in {p.VAR_POSITIONAL, p.VAR_KEYWORD} and p.default is p.empty
+            )
+        )
     )
+    has_varargs = any(
+        p.kind in {p.VAR_KEYWORD, p.VAR_POSITIONAL} for p in params.values()
+    )
+    kwargs = {
+        "type": t,
+        "nullable": nullable,
+        "name": name,
+        "required_keys": required,
+        "items": items,
+        "total": not has_varargs,
+    }
     cls = ObjectConstraints
     if istypeddict(t):
         cls = TypedDictConstraints
-        kwargs.update(type=dict, ttype=t)
+        kwargs.update(type=dict, ttype=t, total=getattr(t, "__total__", bool(required)))
     return cls(**kwargs)  # type: ignore
 
 
@@ -296,8 +310,14 @@ _CONSTRAINT_BUILDER_HANDLERS = TypeMap(
 def get_constraints(
     t: Type[VT], *, nullable: bool = False, name: str = None
 ) -> ConstraintsT:
+    while should_unwrap(t):
+        nullable = nullable or isoptionaltype(t)
+        t = get_args(t)[0]
     if isconstrained(t):
-        return t.__constraints__  # type: ignore
+        c: ConstraintsT = t.__constraints__  # type: ignore
+        if (c.name, c.nullable) != (name, nullable):
+            return dataclasses.replace(c, name=name, nullable=nullable)
+        return c
     if issubclass(t, enum.Enum):
         return _from_enum_type(t, nullable=nullable, name=name)  # type: ignore
     if isnamedtuple(t) or istypeddict(t):
