@@ -3,10 +3,11 @@
 import dataclasses
 from typing import ClassVar, Type, Pattern, Tuple, Optional, Union, Dict, Any, Text
 
-from .. import gen
+from typic import gen, util
 from .common import BaseConstraints, ContextT, ChecksT
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class TextConstraints(BaseConstraints):
     """Specific constraints pertaining to text-like types (`AnyStr` in Python).
@@ -34,11 +35,11 @@ class TextConstraints(BaseConstraints):
     def _build_validator(self, func: gen.Block) -> Tuple[ChecksT, ContextT]:
         # Set up the local env.
         if self.curtail_length is not None:
-            func.l(f"{self.VAL} = {self.VAL}[:{self.curtail_length}]")
+            func.l(f"{self.VALUE} = {self.VALUE}[:{self.curtail_length}]")
         if self.strip_whitespace:
-            func.l(f"{self.VAL} = {self.VAL}.strip()")
+            func.l(f"{self.VALUE} = {self.VALUE}.strip()")
         if {self.min_length, self.max_length} != {None, None}:
-            func.l(f"size = len({self.VAL})")
+            func.l(f"size = len({self.VALUE})")
         # Build the validation.
         checks = []
         context: Dict[str, Any] = {}
@@ -48,7 +49,7 @@ class TextConstraints(BaseConstraints):
             checks.append(f"size >= {self.min_length}")
         if self.regex is not None:
             context.update(__pattern=self.regex)
-            checks.append(f"__pattern.match({self.VAL})")
+            checks.append(f"__pattern.match({self.VALUE})")
 
         return checks, context
 
@@ -64,6 +65,7 @@ class TextConstraints(BaseConstraints):
         return {x: y for x, y in schema.items() if x is not None}
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class StrConstraints(TextConstraints):
     """Constraints specifically for :py:class:`str`."""
@@ -71,6 +73,7 @@ class StrConstraints(TextConstraints):
     type: ClassVar[Type[str]] = str
 
 
+@util.apply_slots
 @dataclasses.dataclass(frozen=True, repr=False)
 class BytesConstraints(TextConstraints):
     """Constraints specifically for :py:class:`bytes`."""
