@@ -12,12 +12,12 @@ from typing import (
     Callable,
     Optional,
     Union,
-    Iterable,
 )
 
 from typic.api import wrap_cls, ObjectT
 from typic.types import freeze
 from .serde.common import SerdeFlags
+from typic.util import apply_slots
 
 _field_slots: Tuple[str, ...] = cast(Tuple[str, ...], dataclasses.Field.__slots__) + (
     "exclude",
@@ -118,6 +118,7 @@ def make_typedclass(
     delay: bool = False,
     strict: bool = False,
     jsonschema: bool = False,
+    slots: bool = False,
     serde: SerdeFlags = None
 ):
     """A convenience function for generating a dataclass with type-coercion.
@@ -143,6 +144,8 @@ def make_typedclass(
         unsafe_hash=unsafe_hash,
         frozen=frozen,
     )
+    if slots:
+        dcls = apply_slots(dcls)
     fields = [
         f if isinstance(f, Field) else Field.from_field(f)
         for f in dataclasses.fields(dcls)
@@ -151,8 +154,7 @@ def make_typedclass(
     exclude = frozenset({f.name for f in fields if f.exclude})
     dcls.__typic_fields__ = (*fields,)
     serde = serde or SerdeFlags()
-    serde.fields = cast(Mapping[str, str], field_names)
-    serde.exclude = cast(Iterable[str], exclude)
+    serde = serde.merge(SerdeFlags(fields=field_names, exclude=exclude))  # type: ignore
     return wrap_cls(
         dcls, delay=delay, strict=strict, jsonschema=jsonschema, serde=serde,
     )
@@ -170,6 +172,7 @@ def klass(
     delay: bool = False,
     strict: bool = False,
     jsonschema: bool = True,
+    slots: bool = False,
     serde: SerdeFlags = None
 ):
     """A convenience decorator for generating a dataclass with type-coercion.
@@ -210,6 +213,7 @@ def klass(
             delay=delay,
             strict=strict,
             jsonschema=jsonschema,
+            slots=slots,
             serde=serde,
         )
 
