@@ -1,8 +1,9 @@
 import datetime
 import functools
 import inspect
+import pathlib
 import re
-from collections import deque, defaultdict
+from collections import deque, defaultdict, abc
 from operator import attrgetter
 from typing import (
     Mapping,
@@ -21,15 +22,16 @@ from typing import (
     Optional,
 )
 
-from pendulum import parse as dateparse
+from pendulum import parse as dateparse, DateTime, instance
 
 from typic import checks, gen, constraints as const
 from typic.strict import STRICT_MODE
 from typic.util import (
     safe_eval,
-    origin as get_origin,
     cached_issubclass,
     cached_signature,
+    get_defname,
+    get_unique_name,
 )
 from typic.common import DEFAULT_ENCODING, VAR_POSITIONAL, VAR_KEYWORD, ObjectT
 from .common import DeserializerT, DeserializerRegistryT, SerdeConfig, Annotation
@@ -98,8 +100,6 @@ class DesFactory:
 
     def __init__(self, resolver: "Resolver"):
         self.resolver = resolver
-        for typ in checks.BUILTIN_TYPES:
-            self.factory(self.resolver.annotation(typ))
 
     def register(self, deserializer: DeserializerT, check: DeserializerT):
         """Register a user-defined coercer.
@@ -111,13 +111,21 @@ class DesFactory:
         """
         self.__USER_DESS.appendleft((check, deserializer))
 
-    def _set_checks(self, func: gen.Block, annotation: "Annotation"):
+    def _set_checks(self, func: gen.Block, anno_name: str, annotation: "Annotation"):
         _checks = []
         _ctx = {}
         if annotation.optional:
             _checks.append(f"{self.VNAME} is None")
         if annotation.has_default:
-            _checks.append(f"{self.VNAME} == __default")
+            if hasattr(annotation.resolved_origin, "equals"):
+                eq = (
+                    f"({self.VNAME}.equals(__default) "
+                    f"if hasattr({self.VNAME}, 'equals') "
+                    f"else {self.VNAME} == __default)"
+                )
+            else:
+                eq = f"{self.VNAME} == __default"
+            _checks.append(f"({self.VTYPE} is {anno_name} and {eq})")
             _ctx["__default"] = annotation.parameter.default
         if _checks:
             check = " or ".join(_checks)
@@ -126,44 +134,55 @@ class DesFactory:
                 b.l(f"return {self.VNAME}")
 
     @staticmethod
-    def _get_des_name(annotation: "Annotation") -> str:
-        return f"deserializer_{hash(annotation)}".replace("-", "_")
+    def _get_name(
+        annotation: "Annotation", constr: Optional["const.ConstraintsT"]
+    ) -> str:
+        return get_defname("deserializer", (annotation, constr))
 
     def _build_date_des(
         self, func: gen.Block, anno_name: str, annotation: "Annotation"
     ):
-        origin = get_origin(annotation.resolved)
+        origin = annotation.resolved_origin
         if issubclass(origin, datetime.datetime):
-            with func.b(f"if issubclass({self.VTYPE}, datetime):") as b:
+            with func.b(
+                f"if isinstance({self.VNAME}, datetime):", datetime=datetime.datetime
+            ) as b:
+                # Use pendulum's helper if possible.
+                if origin is DateTime:
+                    b.l(f"{self.VNAME} = instance({self.VNAME})", instance=instance)
+                else:
+                    b.l(
+                        f"{self.VNAME} = "
+                        f"{anno_name}("
+                        f"{self.VNAME}.year, "
+                        f"{self.VNAME}.month, "
+                        f"{self.VNAME}.day, "
+                        f"{self.VNAME}.hour, "
+                        f"{self.VNAME}.minute, "
+                        f"{self.VNAME}.second, "
+                        f"{self.VNAME}.microsecond, "
+                        f"{self.VNAME}.tzinfo"
+                        f")",
+                    )
+            with func.b(
+                f"elif isinstance({self.VNAME}, date):", date=datetime.date
+            ) as b:
                 b.l(
                     f"{self.VNAME} = "
                     f"{anno_name}("
                     f"{self.VNAME}.year, "
                     f"{self.VNAME}.month, "
-                    f"{self.VNAME}.day,"
-                    f"{self.VNAME}.hour,"
-                    f"{self.VNAME}.minute,"
-                    f"{self.VNAME}.second,"
-                    f"{self.VNAME}.microsecond,"
-                    f"{self.VNAME}.tzinfo,"
+                    f"{self.VNAME}.day"
                     f")",
-                    datetime=datetime.datetime,
-                )
-            with func.b(f"elif issubclass({self.VTYPE}, date):") as b:
-                b.l(
-                    f"{self.VNAME} = "
-                    f"{anno_name}("
-                    f"{self.VNAME}.year, "
-                    f"{self.VNAME}.month, "
-                    f"{self.VNAME}.day)",
-                    date=datetime.date,
                 )
         elif issubclass(origin, datetime.date):
-            with func.b(f"if issubclass({self.VTYPE}, datetime.datetime):") as b:
-                b.l(f"{self.VNAME} = {self.VNAME}.date()", datetime=datetime)
-        with func.b(f"elif issubclass({self.VTYPE}, (int, float)):") as b:
+            with func.b(
+                f"if isinstance({self.VNAME}, datetime):", datetime=datetime.datetime
+            ) as b:
+                b.l(f"{self.VNAME} = {self.VNAME}.date()")
+        with func.b(f"elif isinstance({self.VNAME}, (int, float)):") as b:
             b.l(f"{self.VNAME} = {anno_name}.fromtimestamp({self.VNAME})")
-        with func.b(f"elif issubclass({self.VTYPE}, (str, bytes)):") as b:
+        with func.b(f"elif isinstance({self.VNAME}, (str, bytes)):") as b:
             line = f"{self.VNAME} = dateparse({self.VNAME})"
             # We do the negative assertion here because all datetime objects are
             # subclasses of date.
@@ -174,23 +193,26 @@ class DesFactory:
     def _add_eval(self, func: gen.Block):
         func.l(
             f"_, {self.VNAME} = __eval({self.VNAME}) "
-            f"if issubclass({self.VTYPE}, (str, bytes)) "
+            f"if isinstance({self.VNAME}, (str, bytes)) "
             f"else (False, {self.VNAME})",
             __eval=safe_eval,
         )
-        func.l(f"{self.VTYPE} = type({self.VNAME})")
+        self._add_vtype(func)
 
-    def _add_subclass_check(self, func: gen.Block, anno_name: str):
-        with func.b(f"if issubclass({self.VTYPE}, {anno_name}):") as b:
+    def _add_type_check(self, func: gen.Block, anno_name: str):
+        with func.b(f"if {self.VTYPE} is {anno_name}:") as b:
             b.l(f"{gen.Keyword.RET} {self.VNAME}")
 
+    def _add_vtype(self, func: gen.Block):
+        func.l(f"{self.VTYPE} = {self.VNAME}.__class__")
+
     def _get_default_factory(self, annotation: "Annotation"):
         factory: Union[Type, Callable[..., Any], None] = None
 
         if annotation.args:
             factory_anno = self.resolver.annotation(annotation.args[-1])
-            factory = factory_anno.origin
-            if issubclass(factory_anno.origin, defaultdict):
+            factory = factory_anno.resolved_origin
+            if issubclass(factory_anno.resolved_origin, defaultdict):
                 factory_nested = self._get_default_factory(factory_anno)
 
                 def factory():
@@ -198,14 +220,14 @@ class DesFactory:
 
                 factory.__qualname__ = f"factory({repr(factory_anno.un_resolved)})"  # type: ignore
 
-            if not checks.isbuiltinsubtype(factory_anno.origin):  # type: ignore
+            if not checks.isbuiltinsubtype(factory_anno.resolved_origin):  # type: ignore
 
                 params: Mapping[str, inspect.Parameter] = cached_signature(
-                    factory_anno.origin
+                    factory_anno.resolved_origin
                 ).parameters
                 if not any(p.default is p.empty for p in params.values()):
 
-                    def factory(*, __origin=factory_anno.origin):
+                    def factory(*, __origin=factory_anno.resolved_origin):
                         return __origin()
 
                     factory.__qualname__ = f"factory({repr(factory_anno.un_resolved)})"  # type: ignore
@@ -215,7 +237,7 @@ class DesFactory:
     def _build_builtin_des(
         self, func: gen.Block, anno_name: str, annotation: "Annotation",
     ):
-        origin = get_origin(annotation.resolved)
+        origin = annotation.resolved_origin
         # We should try and evaluate inputs for anything that isn't a subclass of str.
         if issubclass(origin, (Collection, bool, int)) and not issubclass(
             origin, (str, bytes)
@@ -223,14 +245,14 @@ class DesFactory:
             self._add_eval(func)
         # Encode for bytes
         if issubclass(origin, bytes):
-            with func.b(f"if issubclass({self.VTYPE}, str):") as b:
+            with func.b(f"if isinstance({self.VNAME}, str):") as b:
                 b.l(
                     f"{self.VNAME} = {anno_name}("
                     f"{self.VNAME}, encoding={DEFAULT_ENCODING!r})"
                 )
         # Decode for str
         elif issubclass(origin, str):
-            with func.b(f"if issubclass({self.VTYPE}, (bytes, bytearray)):") as b:
+            with func.b(f"if isinstance({self.VNAME}, (bytes, bytearray)):") as b:
                 b.l(f"{self.VNAME} = {self.VNAME}.decode({DEFAULT_ENCODING!r})")
         if issubclass(origin, defaultdict):
             func.namespace[anno_name] = functools.partial(defaultdict, None)
@@ -251,7 +273,7 @@ class DesFactory:
         )
 
     def _build_fromdict_des(self, func: gen.Block, anno_name: str):
-        self._add_subclass_check(func, anno_name)
+        self._add_type_check(func, anno_name)
         self._add_eval(func)
         func.l(f"{self.VNAME} = {anno_name}.from_dict({self.VNAME})")
 
@@ -268,14 +290,7 @@ class DesFactory:
             self._add_eval(func)
 
         fields_deser = {
-            x: self.resolver.resolve(
-                y.resolved,
-                flags=y.serde.flags,
-                name=x,
-                parameter=y.parameter,
-                is_optional=y.optional,
-                is_strict=y.strict,
-            ).transmute
+            x: self.resolver._resolve_from_annotation(y).transmute
             for x, y in annotation.serde.fields.items()
         }
         x = "fields_in[x]"
@@ -288,13 +303,13 @@ class DesFactory:
         self, func: gen.Block, anno_name: str, annotation: "Annotation"
     ):
         self._add_eval(func)
-        with func.b(f"if issubclass({self.VTYPE}, Mapping):", Mapping=Mapping) as b:
+        with func.b(f"if issubclass({self.VTYPE}, Mapping):", Mapping=abc.Mapping) as b:
             if annotation.serde.fields:
                 self._build_typeddict_des(b, anno_name, annotation, eval=False)
             else:
                 b.l(f"{self.VNAME} = {anno_name}(**{self.VNAME})",)
         with func.b(
-            f"elif issubclass({self.VTYPE}, (list, set, frozenset, tuple)):"
+            f"elif isinstance({self.VNAME}, (list, set, frozenset, tuple)):"
         ) as b:
             if annotation.serde.fields:
                 b.l(
@@ -314,7 +329,7 @@ class DesFactory:
             key_type, item_type = args
             key_des = self.resolver.resolve(key_type, flags=annotation.serde.flags)
             item_des = self.resolver.resolve(item_type, flags=annotation.serde.flags)
-        if issubclass(annotation.origin, defaultdict):
+        if issubclass(annotation.resolved_origin, defaultdict):
             factory = self._get_default_factory(annotation)
             func.namespace[anno_name] = functools.partial(defaultdict, factory)
         kd_name = f"{anno_name}_key_des"
@@ -328,7 +343,7 @@ class DesFactory:
                 x = f"{kd_name}(fields_in.get(x, x))"
             # If there is only a field mapping, get the correct name for the field.
             elif annotation.serde.fields_in:
-                x = f"fields_in.get(x, x)"
+                x = "fields_in.get(x, x)"
             # If there are only serializers, get the serialized value
             elif args:
                 x = f"{kd_name}(x)"
@@ -353,68 +368,112 @@ class DesFactory:
             item_des = self.resolver.resolve(item_type, flags=annotation.serde.flags)
             line = (
                 f"{self.VNAME} = "
-                f"{anno_name}({it_name}(x) for x in {anno_name}({self.VNAME}))"
+                f"{anno_name}({it_name}(x) for x in parent({self.VNAME}))"
             )
 
         self._add_eval(func)
         func.l(line, level=None, **{it_name: item_des})
 
+    def _build_path_des(
+        self, func: gen.Block, anno_name: str, annotation: "Annotation",
+    ):
+        self._add_type_check(func, anno_name)
+        func.l(f"{self.VNAME} = {anno_name}({self.VNAME})")
+
     def _build_generic_des(
         self, func: gen.Block, anno_name: str, annotation: "Annotation"
     ):
-        self._add_subclass_check(func, anno_name)
+        serde = annotation.serde
+        resolved = annotation.resolved
+        self._add_type_check(func, anno_name)
         self._add_eval(func)
-        with func.b(f"if issubclass({self.VTYPE}, Mapping):", Mapping=Mapping) as b:
+        # Main branch - we have a mapping for a user-defined class.
+        # This is where the serde configuration comes in.
+        # WINDY PATH AHEAD
+        func.l("# Happy path - deserialize a mapping into the object.")
+        with func.b(f"if issubclass({self.VTYPE}, Mapping):", Mapping=abc.Mapping) as b:
+            # Universal line - transform input to known keys/values.
+            # Specific values may change.
+            def mainline(k, v):
+                return f"{{{k}: {v} for x in fields_in.keys() & {self.VNAME}.keys()}}"
+
+            # The "happy path" - e.g., no guesswork needed.
+            def happypath(k, v, **ns):
+                b.l(f"{self.VNAME} = {anno_name}(**{mainline(k, v)})", **ns)
+
+            # Default X - translate given `x` to known input `x`
             x = "fields_in[x]"
+            # No field name translation needs to happen.
+            if {*serde.fields_in.keys()} == {*serde.fields_in.values()}:
+                x = "x"
+
+            # Default Y - get the given `y` with the given `x`
             y = f"{self.VNAME}[x]"
-            b.l(f"intersection = fields_in.keys() & {self.VNAME}.keys()")
-            with b.b("if intersection:") as nb:
-                nb.l(
-                    f"{self.VNAME} = "
-                    f"{{{x}: {y} for x in fields_in.keys() & {self.VNAME}.keys()}}"
-                )
-            if not self.resolver.seen(annotation.resolved):
-                b.l(
-                    f"bound = __bind({anno_name}, **{self.VNAME})",
-                    __bind=self.resolver.bind,
-                )
-                b.l(f"{self.VNAME} = bound.eval()")
+            # Get the intersection of known input fields and annotations.
+            matched = {*serde.fields_in.values()} & serde.fields.keys()
+            # Happy path! This is a `@typic.al` wrapped class.
+            if self.resolver.known(resolved) or self.resolver.delayed(resolved):
+                happypath(x, y)
+            # Secondary happy path! We know how to deserialize already.
             else:
-                b.l(f"{self.VNAME} = {anno_name}(**{self.VNAME})")
+                fields_in = serde.fields_in
+                if serde.fields and len(matched) == len(serde.fields_in):
+                    desers = {
+                        f: self.resolver._resolve_from_annotation(
+                            serde.fields[f]
+                        ).transmute
+                        for f in matched
+                    }
+                else:
+                    protocols = self.resolver.protocols(annotation.resolved_origin)
+                    fields_in = {x: x for x in protocols}
+                    desers = {f: p.transmute for f, p in protocols.items()}
+                y = f"desers[{x}]({self.VNAME}[x])"
+                happypath(x, y, desers=desers, fields_in=fields_in)
+
+        # Secondary branch - we have some other input for a user-defined class
+        func.l("# Unknown path, just try casting it directly.")
         with func.b(
             f"elif isbuiltinsubtype({self.VTYPE}):",
             isbuiltinsubtype=checks.isbuiltinsubtype,
         ) as b:
             b.l(f"{self.VNAME} = {anno_name}({self.VNAME})")
+        # Final branch - user-defined class for another user-defined class
+        func.l(
+            "# Two user-defined types, "
+            "try to translate the input into the desired output."
+        )
         with func.b("else:") as b:
             b.l(
-                f"{self.VNAME} = tr({self.VNAME}, {anno_name})",
-                tr=self.resolver.translate,
+                f"{self.VNAME} = translate({self.VNAME}, {anno_name})",
+                translate=self.resolver.translate,
             )
 
-    def _build_des(self, annotation: "Annotation",) -> Callable:
-        func_name = self._get_des_name(annotation)
+    def _build_des(self, annotation: "Annotation", func_name: str) -> Callable:
         args = annotation.args
         # Get the "origin" of the annotation.
         # For natives and their typing.* equivs, this will be a builtin type.
         # For SpecialForms (Union, mainly) this will be the un-subscripted type.
         # For custom types or classes, this will be the same as the annotation.
-        anno_name = f"{func_name}_anno"
-        origin = get_origin(annotation.resolved)
+        origin = annotation.resolved_origin
+        anno_name = get_unique_name(origin)
         ns = {
             anno_name: origin,
+            "parent": getattr(origin, "__parent__", origin),
             "issubclass": cached_issubclass,
             **annotation.serde.asdict(),
         }
         with gen.Block(ns) as main:
             with main.f(func_name, main.param(f"{self.VNAME}")) as func:
-                func.l(f"{self.VTYPE} = type({self.VNAME})")
+                self._add_vtype(func)
                 if origin not in self.UNRESOLVABLE:
-                    self._set_checks(func, annotation)
+                    self._set_checks(func, anno_name, annotation)
                     if checks.isdatetype(origin):
                         self._build_date_des(func, anno_name, annotation)
                     elif origin in {Pattern, re.Pattern}:  # type: ignore
                         self._build_pattern_des(func, anno_name)
+                    elif issubclass(origin, pathlib.Path):
+                        self._build_path_des(func, anno_name, annotation)
                     elif not args and checks.isbuiltintype(origin):
                         self._build_builtin_des(func, anno_name, annotation)
                     elif checks.isfromdictclass(origin):
@@ -423,7 +482,7 @@ class DesFactory:
                         self._build_builtin_des(func, anno_name, annotation)
                     elif checks.istypeddict(origin):
                         self._build_typeddict_des(
-                            func, anno_name, annotation, total=origin.__total__
+                            func, anno_name, annotation, total=origin.__total__  # type: ignore
                         )
                     elif checks.istypedtuple(origin) or checks.isnamedtuple(origin):
                         self._build_typedtuple_des(func, anno_name, annotation)
@@ -449,8 +508,8 @@ class DesFactory:
             def des(__val, *, __des=__des):
                 return (*(__des(x) for x in __val),)
 
-            def validator(__val, *, __validator=__validator):
-                return (*(__validator(x) for x in __val),)
+            def validator(value, *, field: str = None, __validator=__validator):  # type: ignore
+                return (*(__validator(x, field=field) for x in value),)
 
         elif anno.parameter.kind == VAR_KEYWORD:
             __des = des
@@ -459,24 +518,21 @@ class DesFactory:
             def des(__val, *, __des=__des):
                 return {x: __des(y) for x, y in __val.items()}
 
-            def validator(__val, *, __validator=__validator):
-                return {x: __validator(y) for x, y in __val.items()}
+            def validator(value, *, field: str = None, __validator=__validator):  # type: ignore
+                return {x: __validator(y, field=field) for x, y in value.items()}
 
         return des, validator
 
     def _finalize_validator(
-        self, des: DeserializerT, constr: Optional["const.ConstraintsT"],
+        self, constr: Optional["const.ConstraintsT"],
     ) -> "const.ValidatorT":
-        def validate(o):
-            return o
+        def validate(value, *, field: str = None):
+            return value
 
         if constr:
             validate = constr.validate  # noqa: F811
 
-        # Special case for type-constraints which should be coerced for validation.
-        if isinstance(constr, const.TypeConstraints) and constr.coerce:
-            validate = des
-        return validate
+        return validate  # type: ignore
 
     def _finalize_deserializer(
         self,
@@ -485,13 +541,13 @@ class DesFactory:
         constr: Optional["const.ConstraintsT"],
     ) -> Tuple[DeserializerT, "const.ValidatorT"]:
         # Determine how to run in "strict-mode"
-        validator = self._finalize_validator(des, constr)
+        validator = self._finalize_validator(constr)
         # Handle *args and **kwargs
         des, validator = self._check_varargs(anno, des, validator)
-        # If we have type constraints, only validate if we're in strict mode.
+        # If we have type constraints, override the deserializer for strict annotations.
         if isinstance(constr, const.TypeConstraints):
             if anno.strict:
-                des = validator
+                des = validator  # type: ignore
         # Otherwise
         else:
             # In strict mode, we validate & coerce if there are constraints
@@ -502,7 +558,7 @@ class DesFactory:
                     return __d(__v(val))
 
             elif anno.strict:
-                des = validator
+                des = validator  # type: ignore
 
         return des, validator
 
@@ -510,7 +566,7 @@ class DesFactory:
         self, annotation: "Annotation", constr: Optional["const.ConstraintsT"] = None
     ) -> Tuple[DeserializerT, "const.ValidatorT"]:
         annotation.serde = annotation.serde or SerdeConfig()
-        key = self._get_des_name(annotation)
+        key = self._get_name(annotation, constr)
         if key in self.__DES_CACHE:
             return self.__DES_CACHE[key]
         deserializer: Optional[DeserializerT] = None
@@ -519,7 +575,7 @@ class DesFactory:
                 deserializer = des
                 break
         if not deserializer:
-            deserializer = self._build_des(annotation)
+            deserializer = self._build_des(annotation, key)
 
         deserializer, validator = self._finalize_deserializer(
             annotation, deserializer, constr
