@@ -2,6 +2,7 @@ import dataclasses
 import functools
 import inspect
 import warnings
+from collections.abc import Callable
 from operator import attrgetter, methodcaller
 from typing import (
     Mapping,
@@ -39,6 +40,7 @@ from .common import (
 )
 from .des import DesFactory
 from .ser import SerFactory
+from .translator import TranslatorFactory
 
 
 _T = TypeVar("_T")
@@ -54,11 +56,15 @@ class Resolver:
     _DYNAMIC = SerFactory._DYNAMIC
 
     def __init__(self):
-        self.__seen = set()
         self.des = DesFactory(self)
         self.ser = SerFactory(self)
         self.binder = Binder(self)
+        self.translator = TranslatorFactory(self)
         self.bind = self.binder.bind
+        for typ in checks.STDLIB_TYPES:
+            self.resolve(typ)
+            self.resolve(Optional[typ])
+            self.resolve(typ, is_optional=True)
 
     def transmute(self, annotation: Type[ObjectT], value: Any) -> ObjectT:
         """Convert a given value `into` the target annotation.
@@ -78,7 +84,7 @@ class Resolver:
             The value to be transmuted
         """
         resolved: SerdeProtocol = self.resolve(annotation)
-        transmuted: ObjectT = resolved(value)
+        transmuted: ObjectT = resolved.transmute(value)  # type: ignore
 
         return transmuted
 
@@ -120,7 +126,7 @@ class Resolver:
         resolved: SerdeProtocol = self.resolve(annotation)
         value = resolved.validate(value)
         if transmute:
-            return resolved.transmute(value)
+            return resolved.transmute(value)  # type: ignore
         return value
 
     def coerce_value(
@@ -134,25 +140,34 @@ class Resolver:
         )
         return self.transmute(annotation, value)
 
-    def seen(self, t: Type) -> bool:
-        return (
-            t in self.__seen
-            or hasattr(t, ORIG_SETTER_NAME)
-            or not getattr(t, "__delayed__", True)
-        )
+    def known(self, t: Type) -> bool:
+        return hasattr(t, ORIG_SETTER_NAME) or hasattr(t, "__delayed__")
+
+    def delayed(self, t: Type) -> bool:
+        return getattr(t, "__delayed__", False)
 
     @functools.lru_cache(maxsize=None)
     def _get_serializer_proto(self, t: Type) -> SerdeProtocol:
+        tname = util.get_name(t)
         if hasattr(t, SERDE_ATTR):
             return getattr(t, SERDE_ATTR)
-        for name, caller in self._DICT_FACTORY_METHODS:
-            if hasattr(t, name):
+        for attr, caller in self._DICT_FACTORY_METHODS:
+            if hasattr(t, attr):
 
                 def serializer(
-                    val, lazy: bool = False, *, __prim=self.primitive, __call=caller
+                    val,
+                    lazy: bool = False,
+                    name: util.ReprT = None,
+                    *,
+                    __prim=self.primitive,
+                    __call=caller,
+                    __tname=tname,
+                    __repr=util.joinedrepr,
                 ):
+                    name = name or __tname
                     return {
-                        __prim(x): __prim(y, lazy=lazy) for x, y in __call(val).items()
+                        __prim(x): __prim(y, lazy=lazy, name=__repr(name, x))
+                        for x, y in __call(val).items()
                     }
 
                 return SerdeProtocol(
@@ -171,7 +186,7 @@ class Resolver:
         serde: SerdeProtocol = self.resolve(t, flags=settings, _des=False)
         return serde
 
-    def primitive(self, obj: Any, lazy: bool = False) -> Any:
+    def primitive(self, obj: Any, lazy: bool = False, name: util.ReprT = None) -> Any:
         """A method for converting an object to its primitive equivalent.
 
         Useful for encoding data to JSON.
@@ -205,12 +220,12 @@ class Resolver:
         >>> typic.primitive(Foo())
         {'bar': 'bar'}
         """
-        t = type(obj)
+        t = obj.__class__
         if checks.isenumtype(t):
             obj = obj.value
-            t = type(obj)
+            t = obj.__class__
         proto: SerdeProtocol = self._get_serializer_proto(t)
-        return proto.primitive(obj, lazy=lazy)
+        return proto.primitive(obj, lazy=lazy, name=name)  # type: ignore
 
     def tojson(
         self, obj: Any, *, indent: int = 0, ensure_ascii: bool = False, **kwargs
@@ -258,10 +273,10 @@ class Resolver:
         >>> typic.tojson(Enum.FOO)
         '"foo"'
         """
-        t = type(obj)
+        t = obj.__class__
         if checks.isenumtype(t):
             obj = obj.value
-            t = type(obj)
+            t = obj.__class__
         proto: SerdeProtocol = self._get_serializer_proto(t)
         return proto.tojson(obj, indent=indent, ensure_ascii=ensure_ascii, **kwargs)
 
@@ -358,7 +373,9 @@ class Resolver:
         # Get the unfiltered args
         args = getattr(non_super, "__args__", None)
         # Set whether this is optional/strict
-        is_optional = is_optional or checks.isoptionaltype(non_super)
+        is_optional = (
+            is_optional or checks.isoptionaltype(non_super) or parameter.default is None
+        )
         is_strict = is_strict or checks.isstrict(non_super) or self.STRICT
         is_static = util.origin(use) not in self._DYNAMIC
         # Determine whether we should use the first arg of the annotation
@@ -384,7 +401,7 @@ class Resolver:
             else SerdeConfig(flags)
         )
 
-        return Annotation(
+        anno = Annotation(
             resolved=use,
             origin=orig,
             un_resolved=annotation,
@@ -394,6 +411,31 @@ class Resolver:
             static=is_static,
             serde=serde,
         )
+        anno.translator = functools.partial(self.translator.factory, anno)  # type: ignore
+        return anno
+
+    @functools.lru_cache(maxsize=None)
+    def _resolve_from_annotation(
+        self, anno: Annotation, _des: bool = True, _ser: bool = True,
+    ) -> SerdeProtocol:
+        # FIXME: Simulate legacy behavior. Should add runtime analysis soon (#95)
+        if anno.origin is Callable:
+            _des, _ser = False, False
+        # Build the deserializer
+        deserializer, validator, constraints = None, None, None
+        if _des:
+            constraints = const.get_constraints(anno.resolved, nullable=anno.optional)
+            deserializer, validator = self.des.factory(anno, constraints)
+        # Build the serializer
+        serializer: Optional[SerializerT] = self.ser.factory(anno) if _ser else None
+        # Put it all together
+        return SerdeProtocol(
+            annotation=anno,
+            deserializer=deserializer,
+            serializer=serializer,
+            constraints=constraints,
+            validator=validator,
+        )
 
     @functools.lru_cache(maxsize=None)
     def resolve(
@@ -451,23 +493,7 @@ class Resolver:
             is_strict=is_strict,
             flags=flags,
         )
-        # Build the deserializer
-        deserializer, validator, constraints = None, None, None
-        if _des:
-            constraints = const.get_constraints(anno.resolved, nullable=anno.optional)
-            deserializer, validator = self.des.factory(anno, constraints)
-        # Build the serializer
-        serializer: Optional[SerializerT] = self.ser.factory(anno) if _ser else None
-        # Put it all together
-        resolved = SerdeProtocol(
-            annotation=anno,
-            deserializer=deserializer,
-            serializer=serializer,
-            constraints=constraints,
-            validator=validator,
-        )
-        # Add it to our tracker for external checks.
-        self.__seen.add(annotation)
+        resolved = self._resolve_from_annotation(anno, _des, _ser)
         return resolved
 
     @functools.lru_cache(maxsize=None)
@@ -499,11 +525,10 @@ class Resolver:
         if not any(
             (inspect.ismethod(obj), inspect.isfunction(obj), inspect.isclass(obj))
         ):
-            obj = type(obj)
+            obj = obj.__class__
 
-        sig = util.cached_signature(obj)
         hints = util.cached_type_hints(obj)
-        params: Mapping[str, inspect.Parameter] = sig.parameters
+        params = util.safe_get_params(obj)
         fields: Mapping[str, dataclasses.Field] = {}
         if dataclasses.is_dataclass(obj):
             fields = {f.name: f for f in dataclasses.fields(obj)}
