@@ -27,7 +27,6 @@ from typing import (
     TypeVar,
     Iterable,
     MutableMapping,
-    Tuple,
     ItemsView,
     KeysView,
     ValuesView,
@@ -35,92 +34,125 @@ from typing import (
 )
 
 from typic import util, checks, gen, types
+from typic.ext import json
 from typic.common import DEFAULT_ENCODING
-from .common import SerializerT, SerdeConfig, Annotation
+from .common import (
+    SerializerT,
+    SerdeConfig,
+    Annotation,
+    Unprocessed,
+    Omit,
+    KT,
+    VT,
+)
+
 
 if TYPE_CHECKING:  # pragma: nocover
     from .resolver import Resolver
 
 
 _T = TypeVar("_T")
-KT = TypeVar("KT")
-VT = TypeVar("VT")
-KVPairT = Tuple[KT, VT]
 
 
 def make_class_serdict(annotation: "Annotation", fields: Mapping[str, SerializerT]):
-    name = f"{util.get_name(annotation.origin)}SerDict"
+    name = f"{util.get_name(annotation.resolved_origin)}SerDict"
     bases = (ClassFieldSerDict,)
     getters = annotation.serde.fields_getters
+    omit = (*annotation.serde.omit_values,)
     if annotation.serde.fields_out:
         fout = annotation.serde.fields_out
         getters = {y: getters[x] for x, y in fout.items()}
         fields = {y: fields[x] for x, y in fout.items()}
+
+    def getitem(self, key):
+        v = dict.__getitem__(self, key)
+        return self.__missing__(key) if v is Unprocessed else v
+
+    if omit:
+
+        def missing(
+            self,
+            item,
+            *,
+            __fields=fields,
+            __getters=getters,
+            __repr=util.joinedrepr,
+            __omit=omit,
+        ):
+            raw = __getters[item](self.instance)
+            if raw not in __omit:
+                self[item] = ret = __fields[item](
+                    raw, lazy=self.lazy, name=__repr(self._name, item)
+                )
+                return ret
+            return Omit
+
+        def iter(self, *, __fields=fields.keys(), Omit=Omit):
+            for k in fields:
+                v = self.__missing__(k)
+                if v is Omit:
+                    continue
+                yield k
+
+    else:
+
+        def missing(  # type: ignore
+            self, item, *, __fields=fields, __getters=getters, __repr=util.joinedrepr,
+        ):
+            self[item] = ret = __fields[item](
+                __getters[item](self.instance),
+                lazy=self.lazy,
+                name=__repr(self._name, item),
+            )
+            return ret
+
+        def iter(self, *, __fields=fields.keys()):  # type: ignore
+            for k in __fields:
+                self.__missing__(k)
+                yield k
+
+    stdlib = json.using_stdlib()
     ns = dict(
         lazy=False,
         getters=getters,
         fields=fields,
         omit=(*annotation.serde.omit_values,),
+        type=annotation.resolved_origin,
+        __missing__=missing,
+        __iter__=iter,
+        stdlib=stdlib,
     )
+    if stdlib:
+        ns["__getitem__"] = getitem
     return type(name, bases, ns)
 
 
-class _Unprocessed:
-    def __repr__(self):
-        return "<unprocessed>"
-
-
-Unprocessed = _Unprocessed()
-
-
-class _Omit:
-    def __repr__(self):
-        return "<omit>"
-
-
-Omit = _Omit()
+class SerializationValueError(ValueError):
+    ...
 
 
 class ClassFieldSerDict(dict):
+    type: Type
     instance: Any
     lazy: bool
     getters: Mapping[str, Callable[[str], Any]]
     fields: Mapping[str, SerializerT]
     omit: Iterable[Any]
+    stdlib: bool
 
-    def __init__(self, instance: Any, lazy: bool = False):
+    __slots__ = ("instance", "lazy", "_name")
+
+    def __init__(
+        self, instance: Any, lazy: bool = False, *, __tfname__: util.ReprT = None,
+    ):
+        self._name = __tfname__
         self.instance = instance
         self.lazy = lazy
-        super().__init__(((x, Unprocessed) for x in self.fields))
+        if self.stdlib:
+            dict.__init__(self, dict.fromkeys(self.fields, Unprocessed))
 
-    def __hash__(self):  # pragma: nocover
-        h = getattr(self.instance, "__hash__", None)
-        return h() if h else None
-
-    def __iter__(self):
-        removals = set()
-        remadd = removals.add
-        pop = self.pop
-        for k in super().__iter__():
-            v = self[k]
-            if v is Omit:
-                remadd(k)
-                continue
-            yield k
-        for rk in removals:
-            pop(rk)
-
-    def __getitem__(self, item) -> Union[VT, _Omit]:
-        value = super().__getitem__(item)
-        if value is Unprocessed:
-            raw = self.getters[item](self.instance)
-            if raw in self.omit:
-                serialized = Omit
-            else:
-                serialized = self.fields[item](raw, lazy=self.lazy)  # type: ignore
-            self[item] = serialized
-            return serialized
-        return value
+    def __hash__(self):
+        return self.instance.__hash__()
 
     def items(self) -> ItemsView[KT, VT]:  # pragma: nocover
         return ItemsView(self)  # type: ignore
@@ -133,14 +165,65 @@ class ClassFieldSerDict(dict):
 
 
 def make_kv_serdict(annotation: "Annotation", kser: SerializerT, vser: SerializerT):
-    name = f"{util.get_name(annotation.origin)}KVSerDict"
+    name = f"{util.get_name(annotation.resolved_origin)}KVSerDict"
     bases = (KVSerDict,)
+    omit = (*annotation.serde.omit_values,)
+
+    def getitem(self, key):
+        v = dict.__getitem__(self, key)
+        return self.__missing__(key) if v is Unprocessed else v
+
+    if omit:
+
+        def missing(self, key, *, __repr=util.collectionrepr, __vser=vser, __omit=omit):
+            if key in self._unprocessed:
+                value = self._unprocessed[key]
+                if value in __omit:
+                    return Omit
+                newv = __vser(  # type: ignore
+                    value, lazy=self.lazy, name=__repr(self._name, key),
+                )
+                self[key] = newv
+                return newv
+            raise KeyError(f"{__repr(self._name, key)!r}")
+
+        def iter(self, *, Omit=Omit, __kser=kser):  # type: ignore
+            for k in self._unprocessed:
+                if self[k] is Omit:
+                    continue
+                yield k
+
+    else:
+
+        def missing(self, key, *, __repr=util.collectionrepr, __vser=vser):  # type: ignore
+            if key in self._unprocessed:
+                newv = vser(  # type: ignore
+                    self._unprocessed[key],
+                    lazy=self.lazy,
+                    name=__repr(self._name, key),
+                )
+                self[key] = newv
+                return newv
+            raise KeyError(f"{__repr(self._name, key)!r}")
+
+        def iter(self, *, __kser=kser):  # type: ignore
+            for k in self._unprocessed:
+                self[k]
+                yield k
+
+    stdlib = json.using_stdlib()
     ns = dict(
+        type=annotation.generic,
         lazy=False,
         kser=staticmethod(kser),
         vser=staticmethod(vser),
-        omit=(*annotation.serde.omit_values,),
+        omit=omit,
+        __missing__=missing,
+        __iter__=iter,
+        stdlib=stdlib,
     )
+    if stdlib:
+        ns["__getitem__"] = getitem
     return type(name, bases, ns)
 
 
@@ -149,36 +232,17 @@ class KVSerDict(dict):
     vser: SerializerT
     lazy: bool
     omit: Iterable[Any]
+    stdlib: bool
 
-    def __init__(self, seq=None, *, lazy: bool = False, **kwargs):
+    __slots__ = ("_unprocessed", "_name")
+
+    def __init__(self, mapping: Mapping, *, lazy: bool = False, __tfname__: util.ReprT):
         kser = self.kser
-        self._unprocessed = {
-            kser(k): v for k, v in dict(seq, **kwargs).items()  # type: ignore
-        }
-        super().__init__(((k, Unprocessed) for k in self._unprocessed))
+        self._name = __tfname__
+        self._unprocessed = {kser(k): v for k, v in mapping.items()}  # type: ignore
         self.lazy = lazy
-
-    def __iter__(self):
-        pop = self.pop
-        removals = set()
-        remadd = removals.add
-        for k in super().__iter__():
-            if self[k] is Omit:
-                remadd(k)
-                continue
-            yield k
-        for rk in removals:
-            pop(rk)
-
-    def __getitem__(self, item) -> Union[VT, _Omit]:
-        value = super().__getitem__(item)
-        if value in self.omit:
-            return Omit
-        if value is Unprocessed:
-            newv = self.vser(self._unprocessed[item], lazy=self.lazy)  # type: ignore
-            self[item] = newv
-            return newv
-        return value
+        if self.stdlib:
+            dict.__init__(self, dict.fromkeys(self._unprocessed, Unprocessed))
 
     def items(self) -> ItemsView[KT, VT]:  # pragma: nocover
         return ItemsView(self)  # type: ignore
@@ -191,7 +255,7 @@ class KVSerDict(dict):
 
 
 def make_serlist(annotation: "Annotation", serializer: SerializerT):
-    name = f"{util.get_name(annotation.origin)}SerList"
+    name = f"{util.get_name(annotation.resolved_origin)}SerList"
     bases = (SerList,)
     ns = dict(
         lazy=False,
@@ -206,16 +270,33 @@ class SerList(list):
     omit: Iterable[Any]
     lazy: bool
 
-    def __init__(self, seq=(), *, lazy: bool = False):
+    __slots__ = ("repr", "_name")
+
+    def __init__(self, seq=(), *, __tfname__: util.ReprT, lazy: bool = False):
+        self.repr = util.collectionrepr
         self.lazy = lazy
+        self._name = __tfname__
         ser = self.serializer
-        super().__init__((ser(x, lazy=lazy) for x in seq))  # type: ignore
+        list.__init__(
+            self,
+            (
+                ser(x, lazy=lazy, name=self.repr(self._name, i))  # type: ignore
+                for i, x in enumerate(seq)
+            ),
+        )
 
     def __setitem__(self, key, value):  # pragma: nocover
-        super().__setitem__(key, self.serializer(value, lazy=self.lazy))
+        super().__setitem__(
+            key,
+            self.serializer(value, lazy=self.lazy, name=self.repr(self._name, key)),
+        )
 
     def append(self, object: _T) -> None:  # pragma: nocover
-        super().append(self.serializer(object, lazy=self.lazy))  # type: ignore
+        super().append(
+            self.serializer(  # type: ignore
+                object, lazy=self.lazy, name=self.repr(self._name, len(self)),
+            )
+        )
 
 
 def _iso(o) -> str:
@@ -281,6 +362,7 @@ class SerFactory:
     _DYNAMIC = frozenset(
         {Union, Any, inspect.Parameter.empty, dataclasses.MISSING, ClassVar}
     )
+    _FNAME = "fname"
 
     def __init__(self, resolver: "Resolver"):
         self.resolver = resolver
@@ -288,7 +370,36 @@ class SerFactory:
 
     @staticmethod
     def _get_name(annotation: "Annotation") -> str:
-        return f"serializer_{util.hexhash(annotation)}"
+        return util.get_defname("serializer", annotation)
+
+    def _check_add_null_check(self, func: gen.Block, annotation: "Annotation"):
+        if annotation.optional:
+            with func.b("if o is None:") as b:
+                b.l(f"{gen.Keyword.RET}")
+
+    def _add_type_check(self, func: gen.Block, annotation: "Annotation"):
+        resolved_name = util.get_name(annotation.resolved)
+        func.l(f"{self._FNAME} = name or {resolved_name!r}")
+        line = "if not tcheck(o.__class__, t):"
+        check: Callable[[Any], bool] = util.cached_issubclass
+        if checks.isbuiltinsubtype(annotation.generic):
+            line = "if not tcheck(o, t):"
+            check = isinstance  # type: ignore
+        with func.b(line, tcheck=check) as b:
+            msg = (
+                f"{{{self._FNAME}}}: type {{inst_tname!r}} "
+                f"is not a subtype of type "
+                f"{util.get_qualname(annotation.generic)!r}. "
+                f"Perhaps this annotation should be "
+                f"Union[{{inst_tname}}, {util.get_qualname(annotation.generic)}]?"
+            )
+            b.l("inst_tname = qualname(o.__class__)")
+            b.l(
+                f"raise err(f{msg!r})",
+                err=SerializationValueError,
+                qualname=util.get_qualname,
+                t=annotation.generic,
+            )
 
     def _build_list_serializer(
         self, func: gen.Block, annotation: "Annotation",
@@ -301,16 +412,16 @@ class SerFactory:
                 annotation.args[0], flags=annotation.serde.flags
             )
             arg_ser = self.factory(arg_a)
-            arg_ser_name = f"arg_ser"
+            arg_ser_name = "arg_ser"
 
             serlist = make_serlist(annotation, arg_ser)
             serlist_name = serlist.__name__
 
             ns = {serlist_name: serlist, arg_ser_name: arg_ser}
-            line = f"{serlist_name}(o, lazy=lazy)"
+            line = f"{serlist_name}(o, lazy=lazy, __tfname__={self._FNAME})"
 
-        if annotation.optional:
-            line = f"o if o is None else ({line})"
+        self._check_add_null_check(func, annotation)
+        self._add_type_check(func, annotation)
         func.l(f"{gen.Keyword.RET} {line}", level=None, **ns)
 
     def _build_key_serializer(
@@ -339,18 +450,21 @@ class SerFactory:
                 kf.l(f"{gen.Keyword.RET} {k}")
         return main.compile(name=name, ns=ns)
 
-    @staticmethod
     def _finalize_mapping_serializer(
-        func: gen.Block, serdict: Type, annotation: "Annotation"
+        self, func: gen.Block, serdict: Type, annotation: "Annotation",
     ):
         serdict_name = serdict.__name__
-        if annotation.optional:
-            with func.b("if o is None:") as b:
-                b.l(f"{gen.Keyword.RET}")
+        self._check_add_null_check(func, annotation)
+        self._add_type_check(func, annotation)
+
         ns: Dict[str, Any] = {serdict_name: serdict}
-        func.l(f"d = {serdict_name}(o, lazy=lazy)", level=None, **ns)
+        func.l(
+            f"d = {serdict_name}(o, lazy=lazy, __tfname__={self._FNAME})",
+            level=None,
+            **ns,
+        )
         # Write the line.
-        line = f"d if lazy else {{**d}}"
+        line = "d if lazy else {**d}"
         func.l(f"{gen.Keyword.RET} {line}")
 
     def _build_dict_serializer(self, func: gen.Block, annotation: "Annotation"):
@@ -386,9 +500,7 @@ class SerFactory:
         self._finalize_mapping_serializer(func, serdict, annotation)
 
     def _compile_enum_serializer(self, annotation: "Annotation",) -> SerializerT:
-        origin: Type[enum.Enum] = cast(
-            Type[enum.Enum], util.origin(annotation.resolved)
-        )
+        origin: Type[enum.Enum] = cast(Type[enum.Enum], annotation.resolved_origin)
         ts = {type(x.value) for x in origin}
         # If we can predict a single type the return the serializer for that
         if len(ts) == 1:
@@ -396,8 +508,10 @@ class SerFactory:
             va = self.resolver.annotation(t, flags=annotation.serde.flags)
             vser = self.factory(va)
 
-            def serializer(o: enum.Enum, *, lazy: bool = False, _vser=vser):
-                return _vser(o.value, lazy=lazy)
+            def serializer(
+                o: enum.Enum, *, lazy: bool = False, name: util.ReprT = None, _vser=vser
+            ):
+                return _vser(o.value, lazy=lazy, name=name)
 
             return serializer
         # Else default to lazy serialization
@@ -411,11 +525,14 @@ class SerFactory:
         ns = {ser_name: ser}
         with gen.Block(ns) as main:
             with main.f(
-                func_name, main.param("o"), main.param("lazy", default=False)
+                func_name,
+                main.param("o"),
+                main.param("lazy", default=False),
+                main.param("name", default=None),
             ) as func:
+                self._check_add_null_check(func, annotation)
+                self._add_type_check(func, annotation)
                 line = f"{ser_name}(o)"
-                if annotation.optional:
-                    line = f"o if o is None else {line}"
                 func.l(f"{gen.Keyword.RET} {line}")
 
         serializer: SerializerT = main.compile(name=func_name, ns=ns)
@@ -445,7 +562,7 @@ class SerFactory:
             return self._serializer_cache[func_name]
 
         serializer: SerializerT
-        origin = util.origin(annotation.resolved)
+        origin = annotation.resolved_origin
         # Lazy shortcut for messy paths (Union, Any, ...)
         if origin in self._DYNAMIC or not annotation.static:
             serializer = self.resolver.primitive
@@ -453,10 +570,22 @@ class SerFactory:
         elif checks.isenumtype(annotation.resolved):
             serializer = self._compile_enum_serializer(annotation)
         # Primitives don't require further processing.
+        # Just check for nullable and the correct type.
         elif origin in self._PRIMITIVES:
+            ns: dict = {}
+            with gen.Block(ns) as main:
+                with main.f(
+                    func_name,
+                    main.param("o"),
+                    main.param("lazy", default=False),
+                    main.param("name", default=None),
+                ) as func:
+                    self._check_add_null_check(func, annotation)
+                    self._add_type_check(func, annotation)
+                    func.l(f"{gen.Keyword.RET} o")
 
-            def serializer(o: _T, lazy: bool = False) -> _T:
-                return o
+            serializer = main.compile(name=func_name, ns=ns)
+            self._serializer_cache[func_name] = serializer
 
         # Defined cases are pre-compiled, but we have to check for optionals.
         elif origin in self._DEFINED:
@@ -473,7 +602,10 @@ class SerFactory:
             ns = {anno_name: origin, **annotation.serde.asdict()}
             with gen.Block(ns) as main:
                 with main.f(
-                    func_name, main.param("o"), main.param("lazy", default=False)
+                    func_name,
+                    main.param("o"),
+                    main.param("lazy", default=False),
+                    main.param("name", default=None),
                 ) as func:
                     # Mapping types need special nested processing as well
                     if not checks.istypeddict(origin) and issubclass(
